package com.fine47.flutter_native_config;

import android.content.Context;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageManager;
import android.util.Log;

import java.util.Set;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/** FlutterNativeConfigPlugin */
@SuppressWarnings("WeakerAccess")
public class FlutterNativeConfigPlugin implements MethodCallHandler {

  private static final String LOG_TAG = "FlutterNativeConfig";

  private static final String[] easyAccess = new String[] {
    "getGMSVersion", "com.google.android.gms.version",
    "getGoogleMapsApiKey", "com.google.android.geo.API_KEY",
    "getFacebookAppId", "com.facebook.sdk.ApplicationId",
    "getDefaultFirebaseNotificationColor", "com.google.firebase.messaging.default_notification_color"
  };

  /** Plugin registration. */
  public static void registerWith(final Registrar registrar) {
    final MethodChannel channel = new MethodChannel(
      registrar.messenger(),
      "noordawod/flutter_native_config"
    );
    channel.setMethodCallHandler(new FlutterNativeConfigPlugin(registrar));
  }

  public static void debugMetadataFor(final Context context) {
    try {
      final ApplicationInfo appInfo = context.getPackageManager().getApplicationInfo(
        context.getPackageName(),
        PackageManager.GET_META_DATA
      );

      if (null != appInfo && null != appInfo.metaData) {
        final Set<String> keys = appInfo.metaData.keySet();
        if (null != keys && !keys.isEmpty()) {
          Log.i(LOG_TAG, "Found " + keys.size() + " keys in manifest:");
          for (final String key : keys) {
            final Object value = appInfo.metaData.get(key);
            Log.i(LOG_TAG, "  " + key + ": " + value);
          }
        } else {
          Log.i(LOG_TAG, "Found no keys in manifest!");
        }
      }
    } catch(PackageManager.NameNotFoundException e) {
      e.printStackTrace();
    }
  }

  private final Registrar registrar;

  private FlutterNativeConfigPlugin(final Registrar registrar) {
    this.registrar = registrar;

    if (BuildConfig.DEBUG) {
      debugMetadataFor(registrar.context());
    }
  }

  @Override
  public void onMethodCall(final MethodCall call, final Result result) {
    if (call.method.equals("getMetadata")) {
      result.success(getMetadata((String) call.argument("name")));
      return;
    }

    int idx = -1;
    while (easyAccess.length > ++idx) {
      final String method = easyAccess[idx++];

      if (call.method.equals(method)) {
        result.success(getMetadata(easyAccess[idx]));
        return;
      }
    }

    result.notImplemented();
  }

  protected String getMetadata(final String name) {
    try {
      final Context context = registrar.context();

      final ApplicationInfo appInfo = context.getPackageManager().getApplicationInfo(
        context.getPackageName(),
        PackageManager.GET_META_DATA
      );

      final String value = appInfo.metaData.getString(name);

      Log.i(LOG_TAG, "Key=" + name + " Value=" + value);

      return value;
    } catch (NullPointerException ignored) {
      // NO-OP.
    } catch (PackageManager.NameNotFoundException ignored) {
      // NO-OP.
    }

    return null;
  }
}
